@@ -9,7 +9,7 @@ from urllib.parse import urlencode
 
 from ..base import DataRequest
 from ..config import SHConfig
-from ..constants import CRS, CustomUrlParam, MimeType, ServiceType, SHConstants
+from ..constants import CRS, CustomUrlParam, MimeType, ResamplingType, ServiceType, SHConstants
 from ..download import DownloadRequest, SentinelHubDownloadClient
 from ..geo_utils import get_image_dimension
 from ..geometry import BBox, Geometry
@@ -40,8 +40,8 @@ class OgcRequest(DataRequest):
     ):
         """
         :param layer: An ID of a layer configured in Sentinel Hub Dashboard. It has to be configured for the same
-            instance ID which will be used for this request. Also the satellite collection of the layer in Dashboard
-            must match the one given by `data_collection` parameter
+            instance ID which will be used for this request. The satellite collection of the layer in Dashboard
+            must also match the one given by `data_collection` parameter
         :type layer: str
         :param bbox: Bounding box of the requested image. Coordinates must be in the specified coordinate reference
             system.
@@ -144,7 +144,7 @@ class OgcRequest(DataRequest):
         the cloud coverage criteria. Most recent acquisition being first in the list.
 
         :return: list of all available Sentinel-2 acquisition times within request's time interval and
-                acceptable cloud coverage.
+            acceptable cloud coverage.
         :rtype: list(datetime.datetime) or [None]
         """
         return OgcImageService(config=self.config).get_dates(self)
@@ -153,7 +153,7 @@ class OgcRequest(DataRequest):
         """Returns iterator over info about all satellite tiles used for the OgcRequest
 
         :return: Iterator of dictionaries containing info about all satellite tiles used in the request. In case of
-                 DataCollection.DEM it returns None.
+            `DataCollection.DEM` it returns None.
         :rtype: Iterator[dict] or None
         """
         return self.wfs_iterator
@@ -181,8 +181,8 @@ class WmsRequest(OgcRequest):
         :param height: height (number of rows) of the returned image (array)
         :type height: int or None
         :param layer: An ID of a layer configured in Sentinel Hub Dashboard. It has to be configured for the same
-            instance ID which will be used for this request. Also the satellite collection of the layer in Dashboard
-            must match the one given by `data_collection` parameter
+            instance ID which will be used for this request. The satellite collection of the layer in Dashboard
+            must also match the one given by `data_collection` parameter
         :type layer: str
         :param bbox: Bounding box of the requested image. Coordinates must be in the specified coordinate reference
             system.
@@ -250,8 +250,8 @@ class WcsRequest(OgcRequest):
             Sentinel-2 bands.
         :type resy: str
         :param layer: An ID of a layer configured in Sentinel Hub Dashboard. It has to be configured for the same
-            instance ID which will be used for this request. Also the satellite collection of the layer in Dashboard
-            must match the one given by `data_collection` parameter
+            instance ID which will be used for this request. The satellite collection of the layer in Dashboard
+            must also match the one given by `data_collection` parameter
         :type layer: str
         :param bbox: Bounding box of the requested image. Coordinates must be in the specified coordinate reference
             system.
@@ -397,14 +397,14 @@ class OgcImageService:
             params["MAXCC"] = 100.0 * request.maxcc
 
         if hasattr(request, "custom_url_params") and request.custom_url_params is not None:
-            params = {**params, **{k.value: str(v) for k, v in request.custom_url_params.items()}}
+            custom_params = request.custom_url_params
 
-            if CustomUrlParam.EVALSCRIPT.value in params:
-                evalscript = params[CustomUrlParam.EVALSCRIPT.value]
-                params[CustomUrlParam.EVALSCRIPT.value] = b64encode(evalscript.encode()).decode()
+            if CustomUrlParam.EVALSCRIPT in custom_params:
+                evalscript = custom_params[CustomUrlParam.EVALSCRIPT]
+                custom_params[CustomUrlParam.EVALSCRIPT] = b64encode(evalscript.encode()).decode()
 
-            if CustomUrlParam.GEOMETRY.value in params:
-                geometry = params[CustomUrlParam.GEOMETRY.value]
+            if CustomUrlParam.GEOMETRY in custom_params:
+                geometry = custom_params[CustomUrlParam.GEOMETRY]
                 crs = request.bbox.crs
 
                 if isinstance(geometry, Geometry):
@@ -416,7 +416,13 @@ class OgcImageService:
                 if geometry.crs is CRS.WGS84:
                     geometry = geometry.reverse()
 
-                params[CustomUrlParam.GEOMETRY.value] = geometry.wkt
+                custom_params[CustomUrlParam.GEOMETRY] = geometry.wkt
+
+            for resampling in (CustomUrlParam.DOWNSAMPLING, CustomUrlParam.UPSAMPLING):
+                if resampling in custom_params:
+                    custom_params[resampling] = ResamplingType(custom_params[resampling]).value
+
+            params.update({k.value: str(v) for k, v in custom_params.items()})
 
         return params
 
@@ -582,7 +588,7 @@ class OgcImageService:
         """Returns iterator over info about all satellite tiles used for the request
 
         :return: Iterator of dictionaries containing info about all satellite tiles used in the request. In case of
-                 DataCollection.DEM it returns None.
+            `DataCollection.DEM` it returns `None`.
         :rtype: Iterator[dict] or None
         """
         return self.wfs_iterator
